<div class="w-panel-upgrade panel">
    <div class="help-block help-warning">
        <svg class="icon icon-warning icon" aria-hidden="true"><use href="#icon-warning"></use></svg>
        You have not setup any tracking for your site.
		<a href="{{ url('wagtailsettings:edit', 'coop', 'analyticssettings', site.pk) }}">
			Add your relevant IDs
		</a> to setup
		<a href="https://developers.google.com/tag-manager/">Google Tag Manager</a> or
		<a href="https://developers.google.com/analytics/devguides/collection/analyticsjs/">Google Analytics</a>
    </div>
</div>
