# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-07-11 23:47
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('coop', '0003_tag_manager_settings'),
    ]

    operations = [
        migrations.AlterField(
            model_name='analyticssettings',
            name='google_tag_manager',
            field=models.CharField(blank=True, help_text='Your Google Tag Manager container ID, e.g GTM-XXXX', max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='analyticssettings',
            name='google_analytics',
            field=models.CharField(blank=True, help_text='Your Google Analytics property ID, e.g UA-XXXXX-Y', max_length=255, null=True),
        ),
    ]
